"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticDeducer = void 0;
const ts = __importStar(require("typescript"));
const path = __importStar(require("path"));
const base_1 = require("@plutolang/base");
const imports_1 = require("./imports");
const constants_1 = require("./constants");
const visit_var_def_1 = require("./visit-var-def");
const visit_expression_1 = require("./visit-expression");
class StaticDeducer extends base_1.core.Deducer {
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    name = require(path.join(__dirname, "../package.json")).name;
    //eslint-disable-next-line @typescript-eslint/no-var-requires
    version = require(path.join(__dirname, "../package.json")).version;
    constructor(args) {
        super(args);
    }
    async deduce(entrypoints) {
        if (entrypoints.length == 0) {
            throw new Error("The entrypoints is empty.");
        }
        const tsconfigPath = path.resolve("./", "tsconfig.json");
        const configFile = ts.readConfigFile(tsconfigPath, ts.sys.readFile);
        const configJson = ts.parseJsonConfigFileContent(configFile.config, ts.sys, "./");
        const archRef = await compile(entrypoints, configJson.options);
        return { archRef };
    }
}
exports.StaticDeducer = StaticDeducer;
async function compile(fileNames, tsOpts) {
    const program = ts.createProgram(fileNames, tsOpts);
    const allDiagnostics = ts.getPreEmitDiagnostics(program);
    // Emit errors
    allDiagnostics.forEach((diagnostic) => {
        if (diagnostic.file) {
            const { line, character } = ts.getLineAndCharacterOfPosition(diagnostic.file, diagnostic.start);
            const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n");
            console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
        }
        else {
            console.log(ts.flattenDiagnosticMessageText(diagnostic.messageText, "\n"));
        }
    });
    const sourceFile = program.getSourceFile(fileNames[0]);
    const checker = program.getTypeChecker();
    const resVarInfos = [];
    const resRelatInfos = [];
    // Iterate through all the nodes in the global area.
    ts.forEachChild(sourceFile, (node) => {
        const kindName = ts.SyntaxKind[node.kind];
        switch (node.kind) {
            case ts.SyntaxKind.VariableStatement: {
                const curVarInfos = (0, visit_var_def_1.visitVariableStatement)(node, checker);
                resVarInfos.push(...curVarInfos);
                break;
            }
            case ts.SyntaxKind.ExpressionStatement: {
                const union = (0, visit_expression_1.visitExpression)(node, checker);
                resVarInfos.push(...union.resourceVarInfos);
                resRelatInfos.push(...union.resourceRelatInfos);
                break;
            }
            case ts.SyntaxKind.ImportDeclaration:
            case ts.SyntaxKind.EndOfFileToken:
            case ts.SyntaxKind.FunctionDeclaration:
                break;
            default: {
                const { line, character } = ts.getLineAndCharacterOfPosition(sourceFile, node.getStart());
                throw new Error(`${sourceFile.fileName} (${line + 1},${character + 1}): Sorry. Pluto doesn't currently support '${kindName}' in the global area. If you need this feature, please feel free to open an issue and let us know.`);
            }
        }
    });
    return buildArchRef(resVarInfos, resRelatInfos);
}
function buildArchRef(resVarInfos, resRelatInfos) {
    const archResources = resVarInfos.map((varInfo) => {
        const resName = varInfo.varName;
        const resType = varInfo.resourceConstructInfo.constructExpression;
        const resLocs = varInfo.resourceConstructInfo.locations.map((loc) => {
            return {
                file: loc.file,
                depth: loc.depth,
                linenum: {
                    start: loc.start.replace(",", "-").replace(/[()]/g, ""),
                    end: loc.end.replace(",", "-").replace(/[()]/g, ""),
                },
            };
        });
        const resParams = varInfo.resourceConstructInfo.parameters?.map((param, idx) => {
            return {
                index: idx,
                name: "unknown",
                value: param.getText(),
            };
        }) ?? [];
        if (resType == constants_1.FN_RESOURCE_TYPE_NAME) {
            resParams?.push({ name: "name", index: 0, value: `"${resName}"` });
        }
        const res = new base_1.arch.Resource(resName, resType, resLocs, resParams);
        const imports = (0, imports_1.genImportStats)(varInfo.resourceConstructInfo.importElements);
        res.addImports(...imports);
        return res;
    });
    const hasFather = new Set();
    const archRelats = resRelatInfos.map((relatInfo) => {
        if (relatInfo.type == base_1.arch.RelatType.CREATE) {
            relatInfo.toVarNames.forEach((name) => hasFather.add(name));
        }
        const fromRes = archResources.find((val) => val.name == relatInfo.fromVarName);
        const toRes = archResources.find((val) => val.name == relatInfo.toVarNames[0]);
        const relatType = relatInfo.type;
        const relatOp = relatInfo.operation;
        const params = relatInfo.parameters.map((param) => {
            return {
                index: param.order,
                name: param.name,
                value: param.resourceName ?? param.expression.getText(),
            };
        });
        return new base_1.arch.Relationship(fromRes, toRes, relatType, relatOp, params);
    });
    const root = new base_1.arch.Resource("App", "Root"); // Resource Root
    archResources
        .filter((res) => !hasFather.has(res.name))
        .forEach((res) => {
        const relat = new base_1.arch.Relationship(root, res, base_1.arch.RelatType.CREATE, "new");
        archRelats.push(relat);
    });
    const archRef = new base_1.arch.Architecture();
    archRef.addResource(root);
    archResources.forEach((res) => archRef.addResource(res));
    archRelats.forEach((relat) => archRef.addRelationship(relat));
    return archRef;
}
//# sourceMappingURL=deducer.js.map