"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const vitest_1 = require("vitest");
const utils_test_1 = require("./utils-test");
const imports_1 = require("./imports");
const dep_resolve_1 = require("./dep-resolve");
function testOnce(code, importStatNum, expectDepNum) {
    const { sourceFile } = (0, utils_test_1.genAnalyzerForInline)(code);
    // Parse the import statements, and build the import store
    const importStore = new imports_1.ImportStore();
    let elemCount = 0;
    sourceFile.forEachChild((node) => {
        if (!typescript_1.default.isImportDeclaration(node)) {
            return;
        }
        const importElems = (0, imports_1.extractImportElements)(sourceFile, node);
        importStore.update(importElems);
        elemCount += importElems.length;
    });
    (0, vitest_1.expect)(elemCount).toBe(importStatNum);
    // Resolve the dependencies
    const depImportElems = (0, dep_resolve_1.resolveImportDeps)(sourceFile, importStore, sourceFile);
    (0, vitest_1.expect)(depImportElems).toHaveLength(expectDepNum);
}
vitest_1.describe.concurrent("named import cases", async () => {
    const importStat = `import { foo } from "lib";`;
    const cases = [
        { code: "foo.access();", importStatNum: 1, expectDepNum: 1 },
        { code: "foo.prop;", importStatNum: 1, expectDepNum: 1 },
        { code: "new foo();", importStatNum: 1, expectDepNum: 1 },
        { code: "new foo.Cls();", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: foo = new foo();", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: foo.Cls = new foo.Cls();", importStatNum: 1, expectDepNum: 1 },
        { code: "(a: number): Promise<foo> => { }", importStatNum: 1, expectDepNum: 1 },
        { code: "(a: number): Promise<foo.Cls> => { }", importStatNum: 1, expectDepNum: 1 },
    ];
    for (const kase of cases) {
        const code = importStat + kase.code;
        vitest_1.test.concurrent(kase.code, async () => testOnce(code, kase.importStatNum, kase.expectDepNum));
    }
});
vitest_1.describe.concurrent("namespace import cases", async () => {
    const importStat = `import foo from "lib";`;
    const cases = [
        { code: "foo.access();", importStatNum: 1, expectDepNum: 1 },
        { code: "foo.prop;", importStatNum: 1, expectDepNum: 1 },
        { code: "new foo();", importStatNum: 1, expectDepNum: 1 },
        { code: "new foo.Cls();", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: foo = new foo();", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: foo.Cls = new foo.Cls();", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: foo.bar.Cls = new foo.bar.Cls();", importStatNum: 1, expectDepNum: 1 },
        { code: "(a: number): Promise<foo> => { }", importStatNum: 1, expectDepNum: 1 },
        { code: "(a: number): Promise<foo.Cls> => { }", importStatNum: 1, expectDepNum: 1 },
        { code: "const obj: any[] = [];", importStatNum: 1, expectDepNum: 0 },
    ];
    for (const kase of cases) {
        const code = importStat + kase.code;
        vitest_1.test.concurrent(kase.code, async () => testOnce(code, kase.importStatNum, kase.expectDepNum));
    }
});
//# sourceMappingURL=dep-resolve.test.js.map