"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.rmSourceFile = exports.genAnalyzerForFile = exports.genAnalyzerForInline = void 0;
const crypto_1 = require("crypto");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const vitest_1 = require("vitest");
// For testing purposes, the task is to generate a TypeScript `ts.SourceFile` from an inline code.
function genAnalyzerForInline(sourceCode) {
    const fileName = "inline.ts";
    const host = {
        getCompilationSettings: () => ({
            target: typescript_1.default.ScriptTarget.ES2022,
            module: typescript_1.default.ModuleKind.CommonJS,
        }),
        getCurrentDirectory: () => process.cwd(),
        getDefaultLibFileName: (options) => typescript_1.default.getDefaultLibFilePath(options),
        getScriptFileNames: () => [fileName],
        getScriptVersion: () => "0",
        getScriptSnapshot: (name) => name === fileName ? typescript_1.default.ScriptSnapshot.fromString(sourceCode) : undefined,
        readFile: () => undefined,
        fileExists: () => true,
    };
    const service = typescript_1.default.createLanguageService(host);
    const program = service.getProgram();
    (0, vitest_1.expect)(program).toBeDefined();
    const sourceFile = program.getSourceFile(fileName);
    (0, vitest_1.expect)(sourceFile).toBeDefined();
    return {
        sourceFile: sourceFile,
        checker: program.getTypeChecker(),
    };
}
exports.genAnalyzerForInline = genAnalyzerForInline;
function genAnalyzerForFile(content) {
    const filename = "testtmp-" + (0, crypto_1.randomUUID)() + ".ts";
    const filepath = path_1.default.join(__dirname, filename);
    fs_1.default.writeFileSync(filepath, content);
    const tsconfigPath = path_1.default.resolve("./", "tsconfig.json");
    const configFile = typescript_1.default.readConfigFile(tsconfigPath, typescript_1.default.sys.readFile);
    const configJson = typescript_1.default.parseJsonConfigFileContent(configFile.config, typescript_1.default.sys, "./");
    // return await compilePluto(filepaths, configJson.options);
    const program = typescript_1.default.createProgram([filepath], configJson.options);
    const sourceFile = program.getSourceFile(filepath);
    const checker = program.getTypeChecker();
    return { sourceFile, checker };
}
exports.genAnalyzerForFile = genAnalyzerForFile;
function rmSourceFile(sourceFile) {
    fs_1.default.rmSync(sourceFile.fileName);
}
exports.rmSourceFile = rmSourceFile;
//# sourceMappingURL=utils-test.js.map